from typing import Type

from pydantic import (
    BaseModel,
    Field,
    create_model,
)

from popug_sdk.conf import settings


class PaginationMetaSchema(BaseModel):
    page: int
    page_size: int
    count: int


class ResponseMetaSchema(BaseModel):
    app: str = settings.project


def get_list_response_schema(
    schema: Type[BaseModel],
    meta_schema: Type[BaseModel] = ResponseMetaSchema,
    suffix: str = "",
) -> Type[BaseModel]:
    name = schema.__name__.removesuffix("Schema")
    paginated_meta_schema = create_model(
        f"Pagination{meta_schema.__name__}",
        __base__=meta_schema,
        pagination=(PaginationMetaSchema, ...),
    )

    return create_model(
        f"{name}{suffix}ListResponseSchema",
        meta=(
            paginated_meta_schema,
            Field(default_factory=paginated_meta_schema),
        ),
        result=(list[schema], Field(default_factory=list)),  # type: ignore
    )


def get_response_schema(
    schema: Type[BaseModel],
    response_meta: Type[BaseModel] = ResponseMetaSchema,
    suffix: str = "",
) -> Type[BaseModel]:
    name = schema.__name__.removesuffix("Schema")

    return create_model(
        f"{name}{suffix}ResponseSchema",
        meta=(response_meta, Field(default_factory=response_meta)),
        result=(schema, ...),
    )



import subprocess

import typer


def test(directory: str = typer.Option("")) -> None:
    exit(
        subprocess.call(
            [
                "python",
                "-m",
                "pytest",
                directory,
                "-s",
            ]
        )
    )



import pika

from popug_sdk.conf.amqp import AMQPSettings


def get_connection_params(_config: AMQPSettings) -> pika.ConnectionParameters:
    credentials = pika.PlainCredentials(
        _config.username,
        _config.password,
    )
    return pika.ConnectionParameters(
        _config.host,
        _config.port,
        _config.virtual_host,
        credentials,
    )



from contextlib import contextmanager
from typing import Generator

from redis import ConnectionPool

from popug_sdk.conf import settings
from popug_sdk.redis.redis_pool import (
    get_redis_pool,
    init_redis_pool,
)


@contextmanager
def redis_pool(config_name: str) -> Generator[ConnectionPool, None, None]:
    init_redis_pool([config_name], settings.redis)
    pool = get_redis_pool(config_name)

    try:
        yield pool
    finally:
        pool.disconnect()



from typing import Any

from fastapi import (
    Depends,
    HTTPException,
)
from fastapi.security import SecurityScopes
from starlette import status

from popug_sdk.auth.dependencies.token import get_token_data


def check_permissions(
    security_scopes: SecurityScopes,
    token_data: dict[str, Any] = Depends(get_token_data),
) -> None:
    token_scopes = token_data.get("scopes", [])

    for scope in security_scopes.scopes:
        if scope not in token_scopes:
            raise HTTPException(
                status_code=status.HTTP_403_FORBIDDEN,
                detail="You don't have permission to access this resource",
            )



import typer
import uvicorn

from popug_sdk.conf import settings


def runserver(
    host: str = typer.Option("0.0.0.0"),
    port: int = typer.Option(settings.app.app_port),
    debug: bool = typer.Option(False),
    reload: bool = typer.Option(False),
    workers: int = typer.Option(1),
) -> None:
    uvicorn.run(
        settings.app.instance,
        host=host,
        port=port,
        debug=debug,
        reload=reload,
        workers=workers,
    )



from collections import deque
from typing import (
    Generic,
    TypeVar,
)

from sqlalchemy.orm import Session

RepoData = TypeVar("RepoData")


class NoContextError(Exception):
    pass


class BaseRepo(Generic[RepoData]):
    def __init__(
        self,
        session: Session,
    ):
        self._session = session
        self._context_stack: deque[RepoData | None] = deque()

    @property
    def is_empty(self) -> bool:
        return not self._context_stack

    def status(self) -> RepoData:
        self._session.flush()

        return self.get()

    def apply(self) -> RepoData:
        result = self.get()
        self._session.commit()

        return result

    def get(self) -> RepoData:
        result = self._context_stack.pop()

        if result is None:
            raise NoContextError

        return result

    def first(self) -> RepoData | None:
        return self._context_stack.pop()

    def _append_context(self, context: RepoData | None) -> None:
        self._context_stack.append(context)



from setuptools import (
    find_packages,
    setup,
)

FILE_NAME = "VERSION"


EXTRAS = [
    "db",
    "amqp",
    "redis",
]


def get_version():
    with open(FILE_NAME) as file:
        return file.read()


def get_base_requirements():
    with open("requirements.txt") as file:
        return file.readlines()


def get_long_description():
    with open("README.md", encoding="utf-8") as file:
        return file.read()


def get_extras_require():
    all_requires = []
    extras_require = {}

    for extra in EXTRAS:
        with open(f"requirements.{extra}.txt") as file:
            requirements = file.readlines()
            extras_require[extra] = requirements
            all_requires.extend(requirements)

    extras_require["all"] = all_requires

    return extras_require


setup(
    name="popug_sdk",
    version=get_version(),
    package_data={"popug_sdk": ["py.typed"]},
    description="Package for uber popug training project",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/Drozdetskiy/popug_jira",
    author="Mikhail Drozdetskiy",
    author_email="m.drozdetskiy.dev@gmail.com",
    packages=find_packages(),
    install_requires=get_base_requirements(),
    extras_require=get_extras_require(),
    include_package_data=True,
    zip_safe=False,
)



from typing import TypedDict

from fastapi import Query

from popug_sdk.constants import DEFAULT_PAGE_SIZE


class PaginationDict(TypedDict):
    page: int
    page_size: int
    count: int


class BasePagination:
    def __init__(
        self,
        page: int = Query(1, ge=1),
        page_size: int = Query(DEFAULT_PAGE_SIZE, ge=1),
    ) -> None:
        self._page = page
        self._page_size = page_size

    @property
    def limit(self) -> int:
        return self._page_size

    @property
    def offset(self) -> int:
        return (self._page - 1) * self._page_size

    def get_params(self, count: int) -> dict[str, PaginationDict]:
        raise NotImplementedError


class PagePagination(BasePagination):
    def get_params(self, count: int) -> dict[str, PaginationDict]:
        return {
            "pagination": {
                "page": self._page,
                "page_size": self._page_size,
                "count": count,
            }
        }
